﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using FlexChartInkDemo.Utils;
using Windows.Storage.Pickers;
using C1.Xaml.Chart;
using Windows.Storage;
using Windows.Foundation.Metadata;
using Windows.UI;
using Windows.UI.ViewManagement;

// The Blank Page item template is documented at https://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace FlexChartInkDemo
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        Symbol symTouchInput = (Symbol)0xEC87;
        Symbol symCompleted = (Symbol)0xE930;
        const string toolTipStart = "Click to draw annotations";
        const string toolTipComplete = "Click to end drawing";
        string toggleToolTipText = toolTipStart;
        private InkShapeConverter converter;
        private EditableAnnotationLayer annotationLayer;

        public MainPage()
        {
            this.InitializeComponent();
            inkCanvas.InkPresenter.InputDeviceTypes = 
                                                Windows.UI.Core.CoreInputDeviceTypes.Mouse |
                                                Windows.UI.Core.CoreInputDeviceTypes.Touch |
                                                Windows.UI.Core.CoreInputDeviceTypes.Pen;

            converter = new InkShapeConverter(inkCanvas.InkPresenter);
            annotationLayer = new EditableAnnotationLayer(flexChart)
            {
                AllowAdd = false,
                AllowMove = true,
                ContentEditor = null
            };
            flexChart.Layers.Add(annotationLayer);

            Window.Current.CoreWindow.KeyDown += (s, args) =>
            {
                if (args.VirtualKey == Windows.System.VirtualKey.Delete && annotationLayer.SelectedAnnotation != null)
                {
                    annotationLayer.Annotations.Remove(annotationLayer.SelectedAnnotation);
                }
            };
        }

        private async void OnToggleClick(object sender, RoutedEventArgs e)
        {
            vm.AllowDraw = !vm.AllowDraw;
            if (vm.AllowDraw)
            {
                Canvas.SetZIndex(inkCanvas, 2);
                toggleToolTipText = toolTipComplete;
                symDrawAnno.Symbol = symCompleted;
                ToolTipService.SetToolTip(btnToggle, toolTipComplete);
                flexChart.ToolTip.Visibility = Visibility.Collapsed;
            }
            else
            {
                UpdateStatus("Please wait while the annotations are being converted",false);
                var annotations = await converter.ProcessInkAsync();
                foreach (var anno in annotations)
                    annotationLayer.Annotations.Add(anno);
                UpdateStatus("Annotations converted successfully!!",true);
                Canvas.SetZIndex(inkCanvas, -1);
                symDrawAnno.Symbol = symTouchInput;
                ToolTipService.SetToolTip(btnToggle, toolTipStart);
                flexChart.ToolTip.Visibility = Visibility.Visible;
            }
        }

        private void OnCleanAnnoClick(object sender, RoutedEventArgs e)
        {
            annotationLayer.Annotations.Clear();
        }

        private async void OnSaveImageClick(object sender, RoutedEventArgs e)
        {
            var picker = new FileSavePicker();
            picker.SuggestedStartLocation = PickerLocationId.PicturesLibrary;
            Enum.GetNames(typeof(ImageFormat)).ToList().ForEach(fmt =>
            {
                picker.FileTypeChoices.Add(fmt.ToString().ToUpper(), new List<string>() { "." + fmt.ToString().ToLower() });
            });
            StorageFile file = await picker.PickSaveFileAsync();
            if (file != null)
            {
                UpdateStatus("Please wait while the image is being saved",false);
                Stream stream = new MemoryStream();
                var extension = file.FileType.Substring(1, file.FileType.Length - 1);
                ImageFormat fmt = (ImageFormat)Enum.Parse(typeof(ImageFormat), extension, true);
                flexChart.SaveImage(stream, fmt);
                using (Stream saveStream = await file.OpenStreamForWriteAsync())
                {
                    stream.CopyTo(saveStream);
                    stream.Seek(0, SeekOrigin.Begin);
                    saveStream.Seek(0, SeekOrigin.Begin);
                    saveStream.Flush();
                    saveStream.Dispose();
                }
                UpdateStatus("Image Saved Successfully!!",true);
            }
        }

        private async void UpdateStatus(string text,bool isComplete)
        {
            //Mobile customization
            if (ApiInformation.IsTypePresent("Windows.UI.ViewManagement.StatusBar"))
            {
                txtStausInfo.Text = string.Empty;
                var statusBar = StatusBar.GetForCurrentView();
                if (statusBar != null)
                {
                    statusBar.BackgroundOpacity = 1;
                    statusBar.BackgroundColor = Colors.Green;
                    statusBar.ForegroundColor = Colors.White;
                    if(isComplete)
                        statusBar.ProgressIndicator.ProgressValue = null;
                    else
                        statusBar.ProgressIndicator.ProgressValue = 0;
                    statusBar.ProgressIndicator.Text = text;
                    statusBar.ShowAsync();
                    statusBar.ProgressIndicator.ShowAsync();
                    await Task.Delay(TimeSpan.FromSeconds(2));
                    await statusBar.ProgressIndicator.HideAsync();
                    await statusBar.HideAsync();
                }
            }
            //PC customization
            else if (ApiInformation.IsTypePresent("Windows.UI.ViewManagement.ApplicationView"))
            {
                txtStausInfo.Text = text;
                await Task.Delay(TimeSpan.FromSeconds(2));
                txtStausInfo.Text = string.Empty;
            }
        }
    }
}
